## Copyright (C) 2021 Armin Auf der Maur, 6230 Ebikon, Switzerland
## arminaufdermaur@yahoo.de
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <https://www.gnu.org/licenses/>.

## Author: Armin Auf der Maur <armin@LAPTOP-UL2NPON7>
## Revised: 2021-11-03
%% Bootstrapping y (outcome, input1) to test for H0, title: H0booty
%% R1dif2rr3: 1 = R, 2 = dif, 3 = rr (0 for hits in the tea tasting experiment)
%% A valid command to run H0booty is e.g.: H0booty("83E.txt","83sc.txt",3);

function [retval] = H0booty(input1, input2, R1dif2rr3 = 1);
N=10000; %for precision change to N = 100000,
rrmax =10; rrmin =0.1; %max min rr
y = load(input1); % outcome
x = load(input2); % treatment
n =length(y); avx =mean(x); avy=mean(y); ns =(n-length(x(x ==0)));
m = fix(N/n); N =m *n;
stx =std(x, 1); sty =std(y, 1); stxy =stx *sty;
R = (mean(x .* y)-avx*avy) /(stxy),
CorTie =-0.01*sign(R)/N;
dif = R *sty*avx*n/(stx*ns),
Rcr=avy*stx/(mean(x)*sty), rr=1+(n/ns)/(-1+Rcr/R),
hit = 0; for (i = 1:n),  if (y(i).-x(i) ==0), ++hit; endif, endfor,
nB4 = n /kurtosis(y), % effective sample size
No=0;j=0; switch(R1dif2rr3),case(1),Ri =zeros(1, N);case (2),difi =zeros(1, N);
case (3),rri =ones(1, N);case (0), hiti =zeros(1, N); endswitch, 
do
mn =randi(n, n); %creates a matrix of n x n integer random numbers 1:n
for (i=1:n)
a =mn(i,:); k = i +n*j; ye =y(a); stye =std(ye, 1);
switch (R1dif2rr3), 
case (0),hj=0;for(m = 1:n),if (ye(m)-x(m)==0),++hj;endif,endfor,hiti(k)=hj;
otherwise, if (stye !=0); stxye =stye *stx; avye =mean(ye); avxye=avx *avye;
xye=mean(ye.*x);Rj=(xye-avxye)/stxye;  else, ++No; continue; endif, endswitch,
switch (R1dif2rr3)
case (1), Ri(k) =Rj; 
case (2), difi(k) = Rj*stye*avx*n/(stx*ns);
case (3), Rcre=avye*stx/(avx*stye);Rc1 =0.99*Rcre; Rc2 =-0.99*Rcre*(ns/(n-ns));
rp=max(min(Rj,Rc1),Rc2); rri(k)=max(min(1+(n/ns)/(-1+Rcre /rp),rrmax),rrmin);
endswitch, endfor, ++j; until (k >=N),
z =[1:N]; if(rem(N, 2)==0), z1=[1:N/2, N-[N/2:N-1]];
else, z1=[1:N/2+0.5, N-[0.5+N/2:N-1]]; endif, No, N,
switch (R1dif2rr3)
case (1)
Ri = sort(Ri);
PRH0 = lookup(Ri, R +CorTie)/N; PRH0 =min(PRH0, 1-PRH0),
meanRi = mean(Ri), medRi = median(Ri), #msste bei 0 sein
semilogy(Ri, z1/N, "r", R, PRH0, "xr")
%semilogy(Ri, z1/N, "c", R, PRH0, "xc")
xlabel ("R")
case (2)
difi = sort(difi);
PdifH0 = lookup(difi, dif +CorTie)/N; PdifH0 =min(PdifH0, 1-PdifH0),
meandif = mean(difi), meddif = median(difi), #msste bei 0 sein
semilogy(difi, z1/N, "r", dif, PdifH0, "xr")
xlabel ("dif (MJ)")
case (3)
rri = sort(rri);
PrrH0 = lookup(rri, rr +CorTie)/N; PrrH0 =min(PrrH0, 1-PrrH0),
meanrr = mean(rri), medrr = median(rri), #medrr msste bei 0 sein
 loglog(rri, z1/N, "r", rr, PrrH0, "xr")
% semilogy(rri, z1/N, "r", rr, PrrH0, "xr")
xlabel ("rr")
case (0)
hit, hiti = sort(hiti); Phit = lookup(hiti, hit +CorTie)/N; 
if(R ==0),Phit =(Phit +lookup(hiti,hit-1/N)/N)/2; endif, Phit=min(Phit,1-Phit),
semilogy(hiti, z1/N, ".r", hit, Phit, "xr");
%semilogy(hiti, z1/N, "y", hit, Phit, "xy");
xlabel ("hits"); ylabel ("P");
endswitch
ylabel ("P"), grid minor on, hold on,
%res = [rr]; 
%res1 = res';
%save rrES.txt res1;
endfunction