## Copyright (C) 2021 Armin Auf der Maur, 6230 Ebikon, Switzerland
## arminaufdermaur@yahoo.de
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <https://www.gnu.org/licenses/>.

## -*- texinfo -*-
## @deftypefn {} {@var{retval} =} H0regr (@var{input1}, @var{input2})
##
## @seealso{}
## @end deftypefn

## Author: Armin Auf der Maur <armin@LAPTOP-UL2NPON7>
## Revised: 2021-11-03
## Fisher's z-test to test for H0. For normal distributions
## A valid command to run H0regr is e.g.: H0z("83E.txt","83sc.txt"); 

function [retval] = H0z(input1, input2);
PL = 0.025; PU = 0.025; N =10000,
; % limits for CI and N may be changed
y=load(input1); x=load(input2); n=length(y);
[R P ci1 ci2] = corrcoef([x,y]); % Octave program
R =R(1, 2),
PtH0 = 0.5 * P(1, 2), % P(t), close to PH0 calculated by z below
z =[1:N]; if(rem(N, 2)==0), z1=[1:N/2, N-[N/2:N-1]];
else, z1=[1:N/2+0.5, N-[0.5+N/2:N-1]]; endif, 
pkg load statistics,
Rz =zeros(1,N);
for (j=[1:N]); pa = j /(N+1); Ra = norminv(pa);
Lo =2 *Ra /sqrt(n -3);
Rz(j) =(exp(Lo) -1)/(exp(Lo) +1);
endfor,
PH0 =lookup(Rz, R)/N; PzH0 =min(PH0, 1 -PH0), % PH0 calculated by z
semilogy(Rz, z1/N, "g", R, PzH0, "xg", R),
ylabel("P"), xlabel("R"), grid minor on, hold on,
endfunction