## Copyright (C) 2020 Armin Auf der Maur, 6230 Ebikon, Switzerland
## arminaufdermaur@yahoo.de
## 
## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

## Author: Armin Auf der Maur <armin@LAPTOP-UL2NPON7>
## Revised: 2021-11-04
## pairwise bootstrap (bivariate, B.Efron1979) to calculate confidence intervals
## R1dif2rr3: 1 = R, 2 = dif, 3 = rr, default =1.
## A valid command to run H1boot is e.g.: H1boot("83E.txt","83sc.txt",3);

function [retval] = H1boot(input1,input2,R1dif2rr3=1);
N=10000; % for more precision N=100000
rrmax =10; rrmin =0.1; %max min rr
y = load(input1); x = load(input2); n =length(y);
m = fix(N/n); N =m *n,
avx =mean(x); avy=mean(y); ns =length(x(x !=0)); stx =std(x, 1); sty=std(y,1);
R = (mean(x .* y)-avx*avy) /(stx *sty), % Pearsons corr coeff
dif =R *sty*avx*n/(stx*ns) % difference (in MJ)
Rcr =avy *stx /avx /sty; rr=1+(n/ns)/(-1+Rcr/R),
CorTie =-0.01*sign(R)/N;
nB4 = n /kurtosis(y), %effective sample size
P = 0; %First P(H0) is calculated by bootstrapping
ns =(n-length(x(x ==0)));
No=0;j=0; switch(R1dif2rr3),case(1), Ri =zeros(1, N); Rb = R*ones(1, N);
case (2),difi =zeros(1, N); difb = dif*ones(1, N);
case (3),rri =ones(1, N); rrb =rr*ones(1, N); endswitch, 
do
ij =randi(n, n); %creates a matrix of n x n integer random numbers 1:n
for (i=1:n)
a =ij(i,:); k = i +n*j; ye =y(a); stye =std(ye, 1); avye =mean(ye);
xb =x(a);stxb =std(xb, 1);stxyb =stye *stxb; avxb =mean(xb);
%xb is the x corresponding to ye for bivariate bootstrap
if (stye !=0); stxye =stye *stx; avxye=avx *avye; 
Rj=(mean(ye.*x)-avxye)/stxye;  else, ++No; continue; endif,
switch (R1dif2rr3),case (1), Ri(k) =Rj;
case (2),difi(k)=Rj*stye*avx*n/(stx*ns);
case (3), Rcre=avye*stx/(avx*stye);Rc1 =0.99*Rcre; Rc2 =-0.99*Rcre*(ns/(n-ns));
rp=max(min(Rj,Rc1),Rc2); rri(k)=max(min(1+(n/ns)/(-1+Rcre /rp),rrmax),rrmin);
endswitch, nsb =length(xb(xb !=0)); 
if (stxyb !=0); avxb=mean(xb); avxyb=avxb *avye;
Ra=(mean(ye.*xb)-avxyb)/stxyb;  else, ++No; continue; endif,
switch (R1dif2rr3)
case (1), Rb(k) =Ra; 
case (2), difb(k) = Ra*stye*avxb*n/(stxb*nsb);
case (3),Rcreb=avye*stxb/(avxb*stye);Rc1 =0.99*Rcreb;Rc2 =-0.99*Rcreb*nsb/(n-nsb);
rp=max(min(Ra,Rc1),Rc2); rrb(k)=max(min(1+(n/nsb)/(-1+Rcreb /rp),rrmax),rrmin);
endswitch, 
endfor, ++j; until (k >=N), %the end of the loops
z =[1:N]; if(rem(N, 2)==0), z1=[1:N/2, N-[N/2:N-1]];
else, z1=[1:N/2+0.5, N-[0.5+N/2:N-1]]; endif,
hold on
switch (R1dif2rr3)
case {1}
Ri = sort(Ri); P = lookup(Ri, R+CorTie)/N; PRH0 = min(P, 1-P),
Rb = sort(Rb); P = lookup(Rb, CorTie)/N; P0H1 =min(P, 1-P),
if (R>=0), R16=Rb(fix(N*0.159)), else, R16=Rb(fix(N*0.841)), endif
semilogy(Rb, z1/N, 'm', R, PRH0, 'xm', R16, 0.159, 'sm')
xlabel ("R")

case {2}
difi = sort(difi); P = lookup(difi, dif+CorTie)/N; PdifH0 = min(P, 1-P),
difb = sort(difb); P = lookup(difb, CorTie)/N; P0H1 = min(P, 1-P),
if (R>=0), dif16=difb(fix(N*0.159)), else, dif16=difb(fix(N*0.841)), endif
semilogy(difb, z1/N,"m", dif,PdifH0,"xm", dif16,0.159,"sm"),
xlabel("dif (MJ/cell)"),

case (3)
rri = sort(rri); P = lookup(rri, rr+CorTie)/N; PrrH0 = min(P, 1-P),
rrb = sort(rrb); P = lookup(rrb, 1+CorTie)/N; P1H1 = min(P, 1-P),
medrr = median(rrb),
if (R>=0), rr16=rrb(fix(N*0.159)), else, rr16=rrb(fix(N*0.841)), endif
% loglog(rrb, z1/N, "r", rr, PrrH0, "xr", rr16, 0.159, "sr")
semilogy(rrb, z1/N, "r", rr, PrrH0, "xr", rr16, 0.159, "sr")
xlabel ("rr")

endswitch
ylabel ("P"), grid minor on

%res = [rrb];
%res1 = res';
%save rrES.txt res1;
endfunction